"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioStream = void 0;
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var AudioStream = /** @class */ (function () {
    function AudioStream(config) {
        if (config === void 0) { config = {}; }
        this.state = {
            playing: false,
            trackInfo: {
                currentTrack: undefined,
                duration: undefined,
                currentTime: undefined
            }
        };
        this.audio = new Audio();
        this.audioEvents = [
            'ended',
            'error',
            'play',
            'playing',
            'pause',
            'timeupdate',
            'canplay',
            'loadedmetadata',
            'loadstart'
        ];
        this.stateChange = new rxjs_1.Subject();
        this.config = config;
    }
    /**
     * Play the Audio Stream
     */
    AudioStream.prototype.play = function () {
        this.audio.play();
    };
    /**
     * Pause the Audio Stream
     */
    AudioStream.prototype.pause = function () {
        this.audio.pause();
    };
    /**
     * Stop the Audio Stream
     */
    AudioStream.prototype.stop = function () {
        this.audio.currentTime = 0;
        this.audio.pause();
    };
    /**
     * Seek audio to particular time
     *
     * @param time
     */
    AudioStream.prototype.seekTo = function (time) {
        this.audio.currentTime = time;
    };
    /**
     * Set the audio stream volume
     *
     * @param volume
     */
    AudioStream.prototype.setVolume = function (volume) {
        this.audio.volume = volume;
    };
    /**
     * Mute the audio stream volume
     *
     * @param mute
     */
    AudioStream.prototype.setMute = function (muted) {
        this.audio.muted = muted;
    };
    /**
     * Set sinkId for this stream
     *
     * @param mute
     */
    AudioStream.prototype.setSinkId = function (deviceId) {
        if (!this.audio.setSinkId) {
            throw new Error('setSinkId is not supported by this browser');
        }
        this.audio.setSinkId(deviceId);
    };
    /**
     * Returns Observable which listens to audio events of audio stream
     */
    AudioStream.prototype.events = function () {
        var _this = this;
        return rxjs_1.Observable.create(function (observer) {
            var handler = function (event) {
                observer.next(event);
                _this.updateStateEvents(event);
            };
            _this.addEvents(_this.audio, _this.audioEvents, handler);
            return function () {
                _this.removeEvents(_this.audio, _this.audioEvents, handler);
            };
        }).pipe(operators_1.share());
    };
    /**
     * Method to get stream of state updates
     */
    AudioStream.prototype.getState = function () {
        return this.stateChange;
    };
    /**
     * load a new track
     */
    AudioStream.prototype.loadTrack = function (src) {
        /** reset the current audio */
        if (this.audio) {
            this.stop();
        }
        this.audio.src = src;
        this.audio.autoplay = this.state.playing || this.config.autoPlay || false;
        this.audio.load();
    };
    /**
     * method to add events listener for audio stream
     * internal implementation, don't use it.
     */
    AudioStream.prototype.addEvents = function (obj, events, handler) {
        events.forEach(function (event) {
            obj.addEventListener(event, handler);
        });
    };
    /**
     * method to remove events listeners for audio stream
     * internal implementation, don't use it.
     */
    AudioStream.prototype.removeEvents = function (obj, events, handler) {
        events.forEach(function (event) {
            obj.removeEventListener(event, handler);
        });
    };
    /**
     * method to update state from audio events
     * internal implementation, don't use it.
     */
    AudioStream.prototype.updateStateEvents = function (event) {
        var trackInfo = this.state.trackInfo;
        switch (event.type) {
            case 'canplay':
                trackInfo.duration = this.audio.duration;
                this.state.playing = false;
                break;
            case 'playing':
                this.state.playing = true;
                break;
            case 'pause':
                this.state.playing = false;
                break;
            case 'timeupdate':
                trackInfo.currentTime = this.audio.currentTime;
                break;
            case 'error':
                this.state.playing = false;
                trackInfo.currentTime = 0;
                trackInfo.duration = 0;
                break;
        }
        this.stateChange.next(this.state);
    };
    /**
     * method to update state from properties
     * internal implementation, don't use it.
     */
    AudioStream.prototype.updateStateProps = function (obj, prop, value) {
        obj[prop] = value;
        this.stateChange.next();
    };
    return AudioStream;
}());
exports.AudioStream = AudioStream;
//# sourceMappingURL=rxjs-audio-stream.js.map